import { __rest } from "tslib";
import React, { useEffect } from 'react';
import { TextInput } from "@patternfly/react-core/dist/esm/components/TextInput/TextInput";
import { Button } from "@patternfly/react-core/dist/esm/components/Button";
import { Select, SelectOption } from "@patternfly/react-core/dist/esm/components/Select";
import ArrowLeftIcon from "@patternfly/react-icons/dist/esm/icons/arrow-left-icon";
import ArrowRightIcon from "@patternfly/react-icons/dist/esm/icons/arrow-right-icon";
import { css } from '@patternfly/react-styles';
import styles from '@patternfly/react-styles/css/components/CalendarMonth/calendar-month';
import commonStyles from '@patternfly/react-styles/css/base/patternfly-common';
import { getUniqueId } from "@patternfly/react-core/dist/esm/helpers/util";
export var Weekday;
(function (Weekday) {
    Weekday[Weekday["Sunday"] = 0] = "Sunday";
    Weekday[Weekday["Monday"] = 1] = "Monday";
    Weekday[Weekday["Tuesday"] = 2] = "Tuesday";
    Weekday[Weekday["Wednesday"] = 3] = "Wednesday";
    Weekday[Weekday["Thursday"] = 4] = "Thursday";
    Weekday[Weekday["Friday"] = 5] = "Friday";
    Weekday[Weekday["Saturday"] = 6] = "Saturday";
})(Weekday || (Weekday = {}));
// Must be numeric given current header design
const yearFormat = (date) => date.getFullYear();
const buildCalendar = (year, month, weekStart) => {
    const selectedDate = new Date(year, month);
    const firstDayOfWeek = new Date(selectedDate);
    firstDayOfWeek.setDate(firstDayOfWeek.getDate() - firstDayOfWeek.getDay() + weekStart);
    // We will always show 6 weeks like google calendar
    // Assume we just want the numbers for now...
    const calendarWeeks = [];
    for (let i = 0; i < 6; i++) {
        const week = [];
        for (let j = 0; j < 7; j++) {
            week.push(new Date(firstDayOfWeek));
            firstDayOfWeek.setDate(firstDayOfWeek.getDate() + 1);
        }
        calendarWeeks.push(week);
    }
    return calendarWeeks;
};
const isSameDate = (d1, d2) => d1.getFullYear() === d2.getFullYear() && d1.getMonth() === d2.getMonth() && d1.getDate() === d2.getDate();
export const CalendarMonth = (_a) => {
    var { date: dateProp = new Date(), locale = undefined, monthFormat = date => date.toLocaleDateString(locale, { month: 'long' }), weekdayFormat = date => date.toLocaleDateString(locale, { weekday: 'narrow' }), longWeekdayFormat = date => date.toLocaleDateString(locale, { weekday: 'long' }), dayFormat = date => date.getDate(), weekStart = 0, // Use the American Sunday as a default
    onChange = () => { }, validators = [() => true], className, onSelectToggle = () => { } } = _a, props = __rest(_a, ["date", "locale", "monthFormat", "weekdayFormat", "longWeekdayFormat", "dayFormat", "weekStart", "onChange", "validators", "className", "onSelectToggle"]);
    const longMonths = [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11].map(monthNum => new Date(1990, monthNum)).map(monthFormat);
    const [isSelectOpen, setIsSelectOpen] = React.useState(false);
    const [focusedDate, setFocusedDate] = React.useState(dateProp);
    const focusRef = React.useRef();
    const [hiddenMonthId] = React.useState(getUniqueId('hidden-month-span'));
    const [focusDate, setFocusDate] = React.useState(true);
    useEffect(() => setFocusedDate(dateProp), [dateProp]);
    useEffect(() => {
        // When using header controls don't move focus
        if (focusDate) {
            if (focusRef.current) {
                focusRef.current.focus();
            }
        }
        else {
            setFocusDate(true);
        }
    }, [focusedDate]);
    const onMonthClick = (toAdd) => {
        const newDate = new Date(focusedDate);
        newDate.setMonth(newDate.getMonth() + toAdd);
        setFocusedDate(newDate);
        setFocusDate(false);
    };
    const onKeyDown = (ev) => {
        const newDate = new Date(focusedDate);
        if (ev.key === 'ArrowUp') {
            newDate.setDate(newDate.getDate() - 7);
        }
        else if (ev.key === 'ArrowRight') {
            newDate.setDate(newDate.getDate() + 1);
        }
        else if (ev.key === 'ArrowDown') {
            newDate.setDate(newDate.getDate() + 7);
        }
        else if (ev.key === 'ArrowLeft') {
            newDate.setDate(newDate.getDate() - 1);
        }
        if (newDate.getTime() !== focusedDate.getTime()) {
            setFocusedDate(newDate);
            setFocusDate(true);
        }
    };
    const today = new Date();
    const calendar = buildCalendar(focusedDate.getFullYear(), focusedDate.getMonth(), weekStart);
    const monthFormatted = monthFormat(focusedDate);
    const yearFormatted = yearFormat(focusedDate);
    return (React.createElement("div", Object.assign({ className: css(styles.calendarMonth, className) }, props),
        React.createElement("div", { className: styles.calendarMonthHeader },
            React.createElement("div", { className: css(styles.calendarMonthHeaderNavControl, styles.modifiers.prevMonth) },
                React.createElement(Button, { variant: "plain", "aria-label": "Previous month", onClick: () => onMonthClick(-1) },
                    React.createElement(ArrowLeftIcon, { "aria-hidden": true }))),
            React.createElement("div", { className: styles.calendarMonthHeaderMonth },
                React.createElement("span", { id: hiddenMonthId, hidden: true }, "Month"),
                React.createElement(Select
                // Max width with "September"
                , { 
                    // Max width with "September"
                    width: "140px", "aria-labelledby": hiddenMonthId, isOpen: isSelectOpen, onToggle: () => {
                        setIsSelectOpen(!isSelectOpen);
                        onSelectToggle(!isSelectOpen);
                    }, onSelect: (_ev, monthNum) => {
                        // When we put CalendarMonth in a Popover we want the Popover's onDocumentClick
                        // to see the SelectOption as a child so it doesn't close the Popover.
                        setIsSelectOpen(false);
                        onSelectToggle(false);
                        const newDate = new Date(focusedDate);
                        newDate.setMonth(Number(monthNum));
                        setFocusedDate(newDate);
                        setFocusDate(false);
                    }, variant: "single", selections: monthFormatted }, longMonths.map((longMonth, index) => (React.createElement(SelectOption, { key: index, value: index, isSelected: longMonth === monthFormatted }, longMonth))))),
            React.createElement("div", { className: styles.calendarMonthHeaderYear },
                React.createElement(TextInput, { "aria-label": "Select year", type: "number", value: yearFormatted, onChange: year => {
                        const newDate = new Date(focusedDate);
                        newDate.setFullYear(+year);
                        setFocusedDate(newDate);
                        setFocusDate(false);
                    } })),
            React.createElement("div", { className: css(styles.calendarMonthHeaderNavControl, styles.modifiers.nextMonth) },
                React.createElement(Button, { variant: "plain", "aria-label": "Next month", onClick: () => onMonthClick(1) },
                    React.createElement(ArrowRightIcon, { "aria-hidden": true })))),
        React.createElement("table", { className: styles.calendarMonthCalendar },
            React.createElement("thead", { className: styles.calendarMonthDays },
                React.createElement("tr", null, calendar[0].map((date, index) => (React.createElement("th", { key: index, className: styles.calendarMonthDay, scope: "col" },
                    React.createElement("span", { className: commonStyles.screenReader }, longWeekdayFormat(date)),
                    React.createElement("span", { "aria-hidden": true }, weekdayFormat(date))))))),
            React.createElement("tbody", { onKeyDown: onKeyDown }, calendar.map((week, index) => (React.createElement("tr", { key: index, className: styles.calendarMonthDatesRow }, week.map((day, index) => {
                const isToday = isSameDate(day, today);
                const isSelected = isSameDate(day, dateProp);
                const isFocused = isSameDate(day, focusedDate);
                const isValid = validators.every(validator => validator(day));
                const isAdjacentMonth = day.getMonth() !== focusedDate.getMonth();
                const dayFormatted = dayFormat(day);
                return (React.createElement("td", { key: index, className: css(styles.calendarMonthDatesCell, isAdjacentMonth && styles.modifiers.adjacentMonth, isToday && styles.modifiers.current, isSelected && styles.modifiers.selected, !isValid && styles.modifiers.disabled) },
                    React.createElement("button", Object.assign({ className: css(styles.calendarMonthDate, isFocused && styles.modifiers.hover), onClick: () => onChange(day), tabIndex: isFocused ? 0 : -1, disabled: !isValid, "aria-label": `${dayFormatted} ${monthFormatted} ${yearFormatted}` }, (isFocused && { ref: focusRef })), dayFormatted)));
            }))))))));
};
//# sourceMappingURL=CalendarMonth.js.map